; DEF.INC =================================================================

        list    F=inhx8m, P=16F84A, R=hex, N=0  ; samovysvtlujc
        __config        3ff9h                   ; fuses
; oscillator frequency: 3.2768MHz => instruction cycle: ~1.22us
; if multiplied by 256 (tmr0) and 16 (prescaler), result is exactly 5 ms
; =========================================================================
w       equ     00h     ; instruction destination is W register
f       equ     01h     ; instruction destination is register itself

; Pin Connections .........................................................
;   RA0 drives anode of 4th digit from left (DISP0)                (out)
;   RA1 drives anode of 3th digit from left (DISP1)                (out)
;   RA2 drives anode of 2nd digit from left (DISP2)                (out)
;   RA3 drives anode of 1st digit from left (DISP3)                (out)
;   RB0 to RB7 display segments and decimal dot, used in multiplex (out)
;   RB0 button #1                                                  (in)
;   RB1 button #2 - start / stop                                   (in)
;   RB2 button #3 - switch HH:MM / MM:SS display                   (in)
;   RB3 button #4 - setup hours                                    (in)
;   RB4 button #5 - setup minutes                                  (in)
;   RB5 button #6 - start display test mode                        (in)
;   RB6 button #7 - next display segment in test mode              (in)
;   RB7 button #8 - stop display test mode, return to clock func   (in)

; Special Function Registers ..............................................
indf    equ     0x00
tmr0    equ     0x01
pc      equ     0x02
status  equ     0x03
fsr     equ     0x04
porta   equ     0x05
portb   equ     0x06
optireg equ     0x81
trisa   equ     0x85
trisb   equ     0x86

; User Registers ..........................................................
hrs     equ     0x10    ; hours, in BCD code
min     equ     0x11    ; minutes, in BCD code
sec     equ     0x12    ; seconds, in BCD code
                        ; hrs, min, sec musi be in this sequence for
                        ; indirect addressing purposes
timer   equ     0x13    ; 1 second delay timer
maxim   equ     0x14    ; maximum value for procedure increm
stat1   equ     0x15    ; system status
                        ; bit0: 0 button not pressed (default)
                        ;       1 button recognized as pressed and held
                        ; bit1: 0 clock runs (default)
                        ;       1 clock stops
                        ; bit2: 0 display HH:MM (default)
                        ;       1 display MM:SS
                        ; bit3: 0 no autorepeat for hours (default)
                        ;       1 autorepeat for hours active
                        ; bit4: 0 no autorepeat for minutes (default)
                        ;       1 autorepeat for minutes active
                        ; bit5: 0 no autorepeat for segments (default)
                        ;       1 autorepeat for segments active
                        ; bit6: 0 real-time clock (default)
                        ;       1 test mode
                        ; bit7: 0 anode register change not allowed (default)
                        ;       1 anode register change allowed
cnt_m   equ     0x16    ; multiplex counter, sequence 0,1,2,3,0,1,2,3,0,1,...
digit   equ     0x17    ; displayed digit register
new     equ     0x18    ; button status register - new values
old     equ     0x19    ; button status register - old values
cnt_d   equ     0x1A    ; delay counter
cnt_a   equ     0x1B    ; autorepeat counter
segmt   equ     0x1C    ; register for segment codes
anode   equ     0x1D    ; register for anodes
wregbak equ     0x1E    ; variable used for context saving of W register
statbak equ     0x1F    ; variable used for context saving of status register

; Bit Definitions .........................................................



carry   equ     00h     ; bit 0 stavovho slova
dcarry  equ     01h     ; bit 1 stavovho slova
nula    equ     02h     ; bit 2 stavovho slova
push    equ     00h     ; bit 0 registru stat1
stop    equ     01h     ; bit 1 registru stat1
hm      equ     02h     ; bit 2 registru stat1
ah      equ     03h     ; bit 3 registru stat1 (autorepeat hodin)
am      equ     04h     ; bit 4 registru stat1 (autorepeat minut)
ad      equ     05h     ; bit 5 registru stat1 (autorepeat diod a segment)
test    equ     06h     ; bit 6 registru stat1 (testovac reim)
zmena   equ     07h     ; bit 7 registru stat1 (povolen/zakzn zmny)
dp      equ     07h     ; bit 7 portu B (desetinn teka)

rp0     equ     5       ; status bit, RAM page select

; Macros ..................................................................
Bank0   macro
        bcf     status, rp0     ; set Bank 0
        endm

Bank1   macro
        bsf     status, rp0     ; set Bank 1
        endm

; END OF DEF.INC ==========================================================
