; SUB.inc =================================================================

; BCD value in W -> 7-segment display code to W
; Value will be placed to port B.
; 0=segment off, 1=segment on
tab1    addwf   pc, f
        retlw   b'11000000'     ; displays 0
        retlw   b'11111001'     ; displays 1
        retlw   b'10100100'     ; displays 2
        retlw   b'10110000'     ; displays 3
        retlw   b'10011001'     ; displays 4
        retlw   b'10010010'     ; displays 5
        retlw   b'10000010'     ; displays 6
        retlw   b'11111000'     ; displays 7
        retlw   b'10000000'     ; displays 8
        retlw   b'10010000'     ; displays 9

; .........................................................................
; Display anode encoding according to cnt_m, anodes are connected to port A.
; 0=display digit on, 1=display digit off
tab2    addwf   pc, f
        retlw   b'11110111'     ; cnt_m=0 - anode 3 active
        retlw   b'11111011'     ; cnt_m=1 - anode 2 active
        retlw   b'11111101'     ; cnt_m=2 - anode 1 active
        retlw   b'11111110'     ; cnt_m=3 - anode 0 active

; .........................................................................
; Initialization of ports and system status. Register hrs, min, sec clear.
Init    movlw   0xFF            ; port A setup
        movwf   porta           ; display drivers off
        clrf    portb           ; port B setup (clear)

        movlw   b'11100000'
        Bank1
        movwf   trisa           ; port A is output
        movlw   00h
        movwf   trisb           ; port B is output
        movlw   b'10000011'     ; tmr0 clock internal, prescaler 1:16
        movwf   optireg         ; pull-ups disabled (required!)
        Bank0
        clrf    stat1           ; initial system status setup
        clrf    cnt_m           ; multiplex counter clear
        clrf    hrs             ; clear hours
        clrf    min             ; clear minutes
        clrf    sec             ; clear seconds
        clrf    new             ; clear button status register - new value
        clrf    old             ; clear button status register - old value
        return                  ; setup is complete

; .........................................................................
; Aktualizan rutina, vstup je adresa registru, jen se m aktualizovat
; (uloen ped volnm do fsr) a hodnota pro peteen (registr maxim).
; Rutina pite jedniku s BCD korekc k registru, na kter ukazuje fsr.
; Jestlie se tento registr po piten jedniky rovn registru maxim,
; rutina tento registr nuluje a nastavuje Z=1.
aktual  movlw   .7
        addwf   indf, f         ; pite do ind 1 s BCD korekc
        btfsc   status, dcarry
        goto    akt_a
        movlw   .6
        subwf   indf, f
akt_a   movf    maxim, w
        xorwf   indf, w
        btfsc   status, nula      ; indf = maxim? (Z=1?)
        clrf    indf            ; ano => nulovan indf
        retlw   00h             ; ne  => maxim

;***************************************************************************
; Rutina zajist ekn 5ms (ne od okamiku voln rutiny, ale od poslednho
; peteen tmr0).
w5ms    comf    tmr0, w
        btfss   status, nula      ; tmr0 = 0ffh? (Z=1?)
        goto    w5ms            ; ne  => ekme
                                ; ano => testujeme peteen
w5msa   btfsc   tmr0, 0         ; tmr0 petekl? (bit0=0?)
        goto    w5msa           ; ne  => ekme na peteen
        retlw   0               ; ano => maxim
;***************************************************************************
; Rutina rozsvt na 5ms jednu slici ze ty monch. Vbr slice se
; provd jednak na zklad bitu hm registru stat1 a jednak na zklad
; hodnoty tae multiplexu cnt_m.
zobraz  movlw   0ffh
        movwf   porta           ; odpojen anod displeje
        movlw   hrs
        movwf   fsr             ; adresa registru hrs do fsr
        btfsc   stat1, hm       ; zobrazujeme hrs a min? (bit=0?)
        incf    fsr, f          ; ne  => v fsr je adresa min
                                ; ano => nic se nedje
        bcf     status, carry     ; nulovn carry pro rotaci
        rrf     cnt_m, w        ; ta multiplexu dlen dvma je ve w
                                ; v carry je 1, byl-li cnt_m lich,
                                ;            0, byl-li cnt_m sud
        iorlw   00h             ; nastavuje bit nula stavovho slova
                                ; nutn - instrukce rrf to nedl
        btfss   status, nula      ; pvodn hodnota cnt_m = 0 nebo 1? (Z=1?)
        incf    fsr, f          ; ne  => fsr ukazuje na nsledujc registr:
                                ;        sec, jsme-li v reimu min, sec
                                ;        min, jsme-li v reimu hrs, min
                                ; ano => nic se nedje
        movf    indf, w         ; vybran registr do w
        movwf   digit           ; a w do digit
        btfss   status, carry     ; cnt_m byl lich? (C=1?)
        swapf   digit, f        ; ne  => prohodme tveice
                                ; ano => beze zmny
        movlw   b'00001111'
        andwf   digit, w        ; zamaskovn, vsledkem je jedna BCD slice
        call    tab1            ; pekdovn BCD slice na segmenty
        movwf   portb           ; kdy segment na port B
        movf    cnt_m, w        ; cnt_m do w
        call    tab2            ; podle cnt_m dekdujeme jednu anodu
        movwf   porta           ; aktivace pslun anode
        bcf     status, carry     ; nulovn carry pro rotaci
        rrf     sec, w          ; vteiny dlen dvma do w
        btfsc   status, carry     ; zobrazuji sudou vteinu? (C=0?)
        goto    zobr_a          ; ne  => teka zhasnuta
                                ; ano => rozsvtme diodu DP2
        movf    cnt_m, w
        xorlw   01h
        btfsc   status, nula      ; cnt_m = 1? (Z=1?)
        bcf     portb, dp       ; ano => rozsvtm diodu DP2
                                ; ne  => nic se nedje
zobr_a  incf    cnt_m, f        ; inkrementujeme cnt_m
        movlw   b'00000011'
        andwf   cnt_m, f        ; zamaskovn, cnt_m te t jen do 3
        retlw   00h             ; konec
;***************************************************************************
; Rutina zajist oeten zkmit pi ten tlatek a zrove zajist na-
; staven pslunch bit registru stat1, ktermi se d stav systmu.
; Rutina te tlatka kadch 20ms (vyuvn registr cnt_m), a je-li tla-
; tko dreno alespo dvakrt za sebou, uzn se stisknut za platn.
tlacit  movf    cnt_m, w
        btfss   status, nula      ; cnt_m = 0? (Z=1?)
        retlw   00h             ; ne  => konec
        movlw   b'11111111'     ; ano => ten stavu tlatek
        movwf   porta           ; odpojen anod displeje
        clrf    portb           ; nulovn mimo jin bitu 0 portu B (spolen
        movlw   b'11111110'     ; rove pro tlatka)
        tris    portb           ; RB0 vstup, ostatn vstupy
        movlw   d'56'
        movwf   cnt_d           ; ta prodlevy = 56
tlac_a  decfsz  cnt_d, f        ; prodleva na ustlen stavu na sbrnici
        goto    tlac_a          ; asi 200s
        movf    portb, w        ; ten portu B (stav tlatek)
                                ; tlatko stisknuto => na pinu je 0
        xorlw   b'11111111'     ; negace stavu tlatek
        andlw   b'00111110'     ; zamaskovn nepotebnch bit
                                ; tlatko stisknuto => bit registru je 1
        movwf   new             ; pepis stavu tlatek do registru new
        clrf    portb           ; nulovn portu B ped pepnutm smru
        movlw   00h
        tris    portb           ; port B opt jako vstup
        btfss   stat1, push     ; tlatko stle dreno? (bit push=1?)
        goto    tlac_d          ; ne  => dal zpracovn
                                ; ano => test, zda-li bylo putno
        clrf    old             ; nulovn old
        movf    new, w
        btfss   status, nula      ; new = 0? (Z=1?)
        goto    tlac_b          ; ne  => testujeme autorepeat
        bcf     stat1, push     ; ano => tlatko putno, bit shozen
        bcf     stat1, ah       ; u nebude autorepeat hodin
        bcf     stat1, am       ; ani autorepeat minut
        retlw   00h             ; konec
tlac_b  decfsz  cnt_a, f        ; ta autorepeatu vypel?
        retlw   00h             ; ne  => konec
        movlw   .12             ; ano => nastaven prodlevy mezi opakovnm
        movwf   cnt_a           ; na 12*20ms = 240ms
        btfss   stat1, ah       ; autorepeat hodin? (bit ah=1?)
        goto    tlac_c          ; ne  => test autorepeatu minut
        movlw   hrs             ; ano => inkrementujeme hodiny
        movwf   fsr
        movlw   24h
        movwf   maxim           ; vbr hodin
        call    aktual          ; aktualizace hodin
        retlw   00h             ; a konec
tlac_c  btfss   stat1, am       ; autorepeat minut? (bit am=1?)
        retlw   00h             ; ne  => konec
        movlw   min             ; inkrementujeme minuty
        movwf   fsr
        movlw   60h
        movwf   maxim           ; vbr minut
        call    aktual          ; aktualizace minut
        retlw   00h             ; a konec
tlac_d  movf    old, w          ; star stav tlatek do w
        andwf   new, w          ; ve w je new and old
        btfss   status, nula      ; new and old = 0? (Z=1?)
        goto    tlac_e          ; ne  => tlatko stisknuto
        movf    new, w          ; ano => pepis novho stavu do starho
        movwf   old
        retlw   00h             ; a konec
tlac_e  bsf     stat1, push     ; tlatko dreno, nastaven bitu
        movlw   d'50'
        movwf   cnt_a           ; prodleva ped autorepeatem 50*20ms = 1000ms
        btfss   new, 1          ; stisknuto tlatko start/stop? (bit1=1?)
        goto    tlac_f          ; ne  => dal testovn
        movlw   b'00000010'     ; ano => negace bitu stop, ostatn beze zmn
        xorwf   stat1, f
        btfsc   stat1, stop     ; hodiny stoj? (bit stop=1?)
        clrf    sec             ; ano => nulovn vtein a konec
        retlw   00h             ; ne  => pouze konec
tlac_f  btfss   new, 2          ; stisknuto tlatko pepnn? (bit2=1?)
        goto    tlac_g          ; ne  => dal testovn
        movlw   b'00000100'     ; ano => negace bitu hm, ostatn beze zmn
        xorwf   stat1, f
        retlw   00h             ; a konec
tlac_g  btfss   new, 3          ; stisknuto tlatko nastaven hrs? (bit3=1?)
        goto    tlac_h          ; ne  => dal testovn
        movlw   hrs             ; ano => inkrementujeme hodiny
        movwf   fsr
        movlw   24h
        movwf   maxim           ; vbr hodin
        call    aktual          ; aktualizace hodin
        bsf     stat1, ah       ; bude autorepeat hodin
        retlw   00h             ; a konec
tlac_h  btfss   new, 4          ; stisknuto tlatko nastaven min? (bit4=1?)
        goto    tlac_i          ; ne  => vstup do testovac sekvence
        movlw   min             ; ano => inkrementujeme minuty
        movwf   fsr
        movlw   60h
        movwf   maxim           ; vbr minut
        call    aktual          ; aktualizace minut
        bsf     stat1, am       ; bude autorepeat minut
        retlw   00h             ; a konec
tlac_i  bsf     stat1, test     ; testovac reim
        retlw   00h             ; konec
;***************************************************************************
; Rutina zajist oeten zkmit pi ten tlatek v testovacm reimu
; Rutina te tlatka kadch 20ms (vyuvn registr cnt_m), a je-li tla-
; tko dreno alespo dvakrt za sebou, uzn se stisknut za platn.
tltest  movf    cnt_m, w
        btfss   status, nula      ; cnt_m = 0? (Z=1?)
        retlw   00h             ; ne  => konec
                                ; ano => ten stavu tlatek
        movlw   b'11111111'
        movwf   porta           ; odpojen anod displeje
        clrf    portb           ; nulovn mimo jin bitu 0 portu B (spolen
        movlw   b'11111110'     ; rove pro tlatka)
        tris    portb           ; RB0 vstup, ostatn vstupy
        movlw   .56
        movwf   cnt_d           ; ta prodlevy = 56
tlte_a  decfsz  cnt_d, f        ; prodleva na ustlen stavu na sbrnici
        goto    tlte_a          ; asi 200s
        movf    portb, w        ; ten portu B (stav tlatek)
                                ; tlatko stisknuto => na pinu je 0
        xorlw   b'11111111'     ; negace stavu tlatek
        andlw   b'11000000'     ; zamaskovn nepotebnch bit
                                ; tlatko stisknuto => bit registru je 1
        movwf   new             ; pepis stavu tlatek do registru new
        clrf    portb           ; nulovn portu B ped pepnutm smru
        movlw   00h
        tris    portb           ; port B opt jako vstup
        btfss   stat1, push     ; tlatko stle dreno? (bit push=1?)
        goto    tlte_c          ; ne  => dal zpracovn
                                ; ano => test, zda-li bylo putno
        clrf    old             ; nulovn old
        movf    new, w
        btfss   status, nula      ; new = 0? (Z=1?)
        goto    tlte_b          ; ne  => testujeme autorepeat
        bcf     stat1, push     ; ano => tlatko putno, bit shozen
        bcf     stat1, ad       ; u nebude autorepeat diod a segment
        retlw   00h             ; konec
tlte_b  decfsz  cnt_a, f        ; ta autorepeatu vypel?
        retlw   00h             ; ne  => konec
        movlw   .10             ; ano => nastaven prodlevy mezi opakovnm
        movwf   cnt_a           ; na 10*20ms = 200ms
        btfss   stat1, ad       ; autorepeat diod a segment? (bit ad=1?)
        retlw   00h             ; ne  => konec
        movlw   0ffh            ; ano => rozsvtme dal diodu nebo segment
        xorwf   segmt, w
        bcf     status, carry     ; nulovn carry pro rotaci pedem
        btfss   status, nula      ; segmt = 0ffh (Z=1?)
        bsf     status, carry     ; ne  => carry = 1
                                ; ano => carry = 0 (odminula)
        btfss   status, nula      ; segmt = 0ffh (Z=1?)
        bsf     stat1, zmena    ; ne  => zmna povolena
                                ; ano => bit nezmnn
        rlf     segmt, f        ; vbr dal segmt nebo segmentu
        retlw   00h             ; a konec
tlte_c  movf    old, w          ; star stav tlatek do w
        andwf   new, w          ; ve w je new and old
        btfss   status, nula      ; new and old = 0? (Z=1?)
        goto    tlte_d          ; ne  => tlatko stisknuto
        movf    new, w          ; ano => pepis novho stavu do starho
        movwf   old
        retlw   00h             ; a konec
tlte_d  bsf     stat1, push     ; tlatko dreno, nastaven bitu
        movlw   .50
        movwf   cnt_a           ; prodleva ped autorepeatem 50*20ms = 1000ms
        btfss   new, 6          ; stisknuto tlatko pepnn? (bit6=1?)
        goto    tlte_e          ; ne  => stisknuto tlatko konec test
        movlw   0ffh            ; ano => rozsvtme dal diodu nebo segment
        xorwf   segmt, w
        bcf     status, carry     ; nulovn carry pro rotaci pedem
        btfss   status, nula      ; segmt = 0ffh (Z=1?)
        bsf     status, carry     ; ne  => carry = 1
                                ; ano => carry = 0 (odminula)
        btfss   status, nula      ; segmt = 0ffh (Z=1?)
        bsf     stat1, zmena    ; ne  => zmna povolena
                                ; ano => bit nezmnn
        rlf     segmt, f        ; vbr dal segmt nebo segmentu
        bsf     stat1, ad       ; bude autorepeat diod a segment
        retlw   00h             ; a konec
tlte_e  movlw   b'11111111'     ; nastaven portu A
        movwf   porta           ; budie zaveny => nic nesvt
        clrf    portb           ; nulovn portu B
        bcf     stat1, test     ; reim relnho asu
        bcf     stat1, push     ; tlatko putno, bit shozen
        bcf     stat1, ad       ; nebude autorepeat diod a segment
        bcf     stat1, zmena    ; zmna zakzna (pro jistotu)
        clrf    cnt_a           ; nulovn tae autorepeatu
        retlw   00h             ; konec

; END OF SUB.INC ==========================================================
